#version 300 es

// We need to conditionally check for the availablity of GL_OES_EGL_image_external_essl3. Some devices require it,
// some devices have merely deprecated GL_OES_EGL_image_external, and some devices don't know GL_OES_EGL_image_external_essl3 exists.

#if __VERSION__ >= 300 && defined(GL_OES_EGL_image_external_essl3)
#extension GL_OES_EGL_image_external_essl3 : require
#else
#extension GL_OES_EGL_image_external : require
#endif
precision mediump float;
precision highp int;

layout(std140) uniform FragBlock
{
    vec4 fillColor;
    vec2 crop;
    float opacity;
} fragUniforms;

layout(std140) uniform YCbCrMatrixConstants
{
    float kB;
    float kR;
    float kG;
    float kCb;
    float kCr;
    float kCbG;
    float kCrG;
} yCbCrMatrixConstants;

uniform highp sampler2D texture0;
uniform highp sampler2D texture1;

in vec2 textureCoord;
layout(location = 0) out vec4 FragColor;

float calculateTextureBlend()
{
    return ((step(0.0, textureCoord.x) * step(0.0, textureCoord.y)) * (1.0 - step(fragUniforms.crop.x, textureCoord.x))) * (1.0 - step(fragUniforms.crop.y, textureCoord.y));
}

void main()
{
    float t = calculateTextureBlend();
    float Y = texture(texture0, textureCoord).x;
    float Cb = texture(texture1, vec2(textureCoord.x * 0.5, textureCoord.y)).x - 0.5;
    float Cr = texture(texture1, vec2((textureCoord.x * 0.5) + 0.5, textureCoord.y)).x - 0.5;
    float r = clamp(Y + (yCbCrMatrixConstants.kCr * Cr), 0.0, 1.0);
    float g = clamp((Y - (yCbCrMatrixConstants.kCrG * Cr)) - (yCbCrMatrixConstants.kCbG * Cb), 0.0, 1.0);
    float b = clamp(Y + (yCbCrMatrixConstants.kCb * Cb), 0.0, 1.0);
    FragColor = mix(fragUniforms.fillColor, vec4(r, g, b, 1.0), vec4(t)) * vec4(1.0, 1.0, 1.0, fragUniforms.opacity);
}

